@@ -27,10 +27,8 @@ from gi.repository import Gtk
 from gi.repository import Pango
 
 from pitivi.configure import get_ui_dir
-from pitivi.effects import AUDIO_EFFECT
 from pitivi.effects import EffectsPropertiesManager
 from pitivi.effects import HIDDEN_EFFECTS
-from pitivi.effects import VIDEO_EFFECT
 from pitivi.undo.timeline import CommitTimelineFinalizingAction
 from pitivi.utils.loggable import Loggable
 from pitivi.utils.ui import disable_scroll
@@ -105,6 +103,7 @@ class EffectProperties(Gtk.Expander, Loggable):
 
     Attributes:
         app (Pitivi): The app.
+        clip (GES.Clip): The clip being configured.
     """
 
     # pylint: disable=too-many-statements
@@ -119,8 +118,7 @@ class EffectProperties(Gtk.Expander, Loggable):
 
         self._project = None
         self._selection = None
-        self.selected_effects = []
-        self.clips = []
+        self.clip = None
         self._effect_config_ui = None
         self.effects_properties_manager = EffectsPropertiesManager(app)
         self.clip_properties = clip_properties
@@ -224,7 +222,7 @@ class EffectProperties(Gtk.Expander, Loggable):
         effects_actions_group = Gio.SimpleActionGroup()
         self.treeview.insert_action_group("clipproperties-effects", effects_actions_group)
         buttons_box.insert_action_group("clipproperties-effects", effects_actions_group)
-        self.app.shortcuts.register_group("clipproperties-effects", _("Clip Effects"))
+        self.app.shortcuts.register_group("clipproperties-effects", _("Clip Effects"), position=60)
 
         self.remove_effect_action = Gio.SimpleAction.new("remove-effect", None)
         self.remove_effect_action.connect("activate", self._removeEffectCb)
@@ -252,41 +250,47 @@ class EffectProperties(Gtk.Expander, Loggable):
         if project:
             self._selection = project.ges_timeline.ui.selection
             self._selection.connect('selection-changed', self._selectionChangedCb)
-            self.selected_effects = self._selection.getSelectedEffects()
         self.__updateAll()
 
     def _selectionChangedCb(self, selection):
-        for clip in self.clips:
-            clip.disconnect_by_func(self._trackElementAddedCb)
-            clip.disconnect_by_func(self._trackElementRemovedCb)
+        if self.clip:
+            self.clip.disconnect_by_func(self._trackElementAddedCb)
+            self.clip.disconnect_by_func(self._trackElementRemovedCb)
+            for track_element in self.clip.get_children(recursive=True):
+                if isinstance(track_element, GES.BaseEffect):
+                    self._disconnect_from_track_element(track_element)
 
-        self.selected_effects = selection.getSelectedEffects()
-
-        if selection:
-            self.clips = list(selection.selected)
-            for clip in self.clips:
-                clip.connect("child-added", self._trackElementAddedCb)
-                clip.connect("child-removed", self._trackElementRemovedCb)
-            self.show()
-        else:
-            self.clips = []
-            self.hide()
+        clips = list(selection.selected)
+        self.clip = clips[0] if len(clips) == 1 else None
+        if self.clip:
+            self.clip.connect("child-added", self._trackElementAddedCb)
+            self.clip.connect("child-removed", self._trackElementRemovedCb)
+            for track_element in self.clip.get_children(recursive=True):
+                if isinstance(track_element, GES.BaseEffect):
+                    self._connect_to_track_element(track_element)
         self.__updateAll()
 
     def _trackElementAddedCb(self, unused_clip, track_element):
         if isinstance(track_element, GES.BaseEffect):
-            selec = self._selection.getSelectedEffects()
-            self.selected_effects = selec
+            self._connect_to_track_element(track_element)
             self.__updateAll()
             for path, row in enumerate(self.storemodel):
                 if row[COL_TRACK_EFFECT] == track_element:
                     self.treeview_selection.select_path(path)
                     break
 
+    def _connect_to_track_element(self, track_element):
+        track_element.connect("notify::active", self._notify_active_cb)
+
+    def _disconnect_from_track_element(self, track_element):
+        track_element.disconnect_by_func(self._notify_active_cb)
+
+    def _notify_active_cb(self, unused_track_element, unused_param_spec):
+        self._updateTreeview()
+
     def _trackElementRemovedCb(self, unused_clip, track_element):
         if isinstance(track_element, GES.BaseEffect):
-            selec = self._selection.getSelectedEffects()
-            self.selected_effects = selec
+            self._disconnect_from_track_element(track_element)
             self.__updateAll()
 
     def _removeEffectCb(self, unused_action, unused_param):
@@ -295,51 +299,26 @@ class EffectProperties(Gtk.Expander, Loggable):
             # Cannot remove nothing,
             return
         effect = self.storemodel.get_value(selected[1], COL_TRACK_EFFECT)
+        selection_path = self.storemodel.get_path(selected[1])
+        # Preserve selection in the tree view.
+        next_selection_index = selection_path.get_indices()[0]
+        effect_count = self.storemodel.iter_n_children()
+        if effect_count - 1 == next_selection_index:
+            next_selection_index -= 1
         self._removeEffect(effect)
+        if next_selection_index >= 0:
+            self.treeview_selection.select_path(next_selection_index)
 
     def _removeEffect(self, effect):
-        pipeline = self._project.ges_timeline.get_parent()
-        with self.app.action_log.started("remove effect", CommitTimelineFinalizingAction(pipeline)):
+        pipeline = self._project.pipeline
+        with self.app.action_log.started("remove effect",
+                                         finalizing_action=CommitTimelineFinalizingAction(pipeline),
+                                         toplevel=True):
             self.__remove_configuration_widget()
             self.effects_properties_manager.cleanCache(effect)
             effect.get_parent().remove(effect)
-            pipeline.commit_timeline()
         self._updateTreeview()
 
-    def addEffectToClip(self, clip, factory_name, priority=None):
-        """Adds the specified effect if it can be applied to the clip."""
-
-        model = self.treeview.get_model()
-        media_type = self.app.effects.getInfo(factory_name).media_type
-        for track_element in clip.get_children(False):
-            track_type = track_element.get_track_type()
-            if track_type == GES.TrackType.AUDIO and media_type == AUDIO_EFFECT or \
-                    track_type == GES.TrackType.VIDEO and media_type == VIDEO_EFFECT:
-                # Actually add the effect
-                pipeline = self._project.ges_timeline.get_parent()
-                with self.app.action_log.started("add effect",
-                                                 CommitTimelineFinalizingAction(pipeline)):
-                    effect = GES.Effect.new(bin_description=factory_name)
-                    clip.add(effect)
-                    if priority is not None and priority < len(model):
-                        clip.set_top_effect_priority(effect, priority)
-                pipeline.commit_timeline()
-                break
-
-    def addEffectToCurrentSelection(self, factory_name):
-        """Adds an effect to the current selection.
-
-        Args:
-            factory_name (str): The name of the GstElementFactory for creating
-                the effect.
-        """
-        if not self.clips or len(self.clips) > 1:
-            return
-        clip = self.clips[0]
-        # Checking that this effect can be applied on this track object
-        # Which means, it has the corresponding media_type
-        self.addEffectToClip(clip, factory_name)
-
     # pylint: disable=too-many-arguments
     def _dragMotionCb(self, unused_tree_view, unused_drag_context, unused_x, unused_y, unused_timestamp):
         self.debug(
@@ -353,23 +332,29 @@ class EffectProperties(Gtk.Expander, Loggable):
 
     # pylint: disable=too-many-arguments
     def _dragDataReceivedCb(self, treeview, drag_context, x, y, selection_data, unused_info, timestamp):
-        if not self.clips or len(self.clips) > 1:
+        if not self.clip:
             # Indicate that a drop will not be accepted.
             Gdk.drag_status(drag_context, 0, timestamp)
             return
-        clip = self.clips[0]
         dest_row = treeview.get_dest_row_at_pos(x, y)
         if drag_context.get_suggested_action() == Gdk.DragAction.COPY:
             # An effect dragged probably from the effects list.
             factory_name = str(selection_data.get_data(), "UTF-8")
             drop_index = self.__get_new_effect_index(dest_row)
-            self.addEffectToClip(clip, factory_name, drop_index)
+            effect_info = self.app.effects.getInfo(factory_name)
+            pipeline = self._project.pipeline
+            with self.app.action_log.started("add effect",
+                                             finalizing_action=CommitTimelineFinalizingAction(pipeline),
+                                             toplevel=True):
+                effect = self.clip.ui.add_effect(effect_info)
+                if effect:
+                    self.clip.set_top_effect_priority(effect, drop_index)
         elif drag_context.get_suggested_action() == Gdk.DragAction.MOVE:
             # An effect dragged from the same treeview to change its position.
             # Source
             source_index, drop_index = self.__get_move_indexes(
                 dest_row, treeview.get_model())
-            self.__move_effect(clip, source_index, drop_index)
+            self.__move_effect(self.clip, source_index, drop_index)
 
         drag_context.finish(True, False, timestamp)
 
@@ -413,10 +398,10 @@ class EffectProperties(Gtk.Expander, Loggable):
         effect = effects[source_index]
         pipeline = self._project.ges_timeline.get_parent()
         with self.app.action_log.started("move effect",
-                                         CommitTimelineFinalizingAction(pipeline)):
+                                         finalizing_action=CommitTimelineFinalizingAction(pipeline),
+                                         toplevel=True):
             clip.set_top_effect_priority(effect, drop_index)
 
-        pipeline.commit_timeline()
         new_path = Gtk.TreePath.new()
         new_path.append_index(drop_index)
         self.__updateAll(path=new_path)
@@ -436,12 +421,15 @@ class EffectProperties(Gtk.Expander, Loggable):
 
     def _effectActiveToggleCb(self, cellrenderertoggle, path):
         _iter = self.storemodel.get_iter(path)
-        tck_effect = self.storemodel.get_value(_iter, COL_TRACK_EFFECT)
-        with self.app.action_log.started("change active state"):
-            tck_effect.set_active(not tck_effect.is_active())
-            cellrenderertoggle.set_active(tck_effect.is_active())
-            self._updateTreeview()
-            self._project.ges_timeline.commit()
+        effect = self.storemodel.get_value(_iter, COL_TRACK_EFFECT)
+        pipeline = self._project.ges_timeline.get_parent()
+        with self.app.action_log.started("change active state",
+                                         finalizing_action=CommitTimelineFinalizingAction(pipeline),
+                                         toplevel=True):
+            effect.props.active = not effect.props.active
+        # This is not strictly necessary, but makes sure
+        # the UI reflects the current status.
+        cellrenderertoggle.set_active(effect.is_active())
 
     def _expandedCb(self, unused_expander, unused_params):
         self.__updateAll()
@@ -460,7 +448,7 @@ class EffectProperties(Gtk.Expander, Loggable):
         return True
 
     def __updateAll(self, path=None):
-        if len(self.clips) == 1:
+        if self.clip:
             self.show()
             self._infobar.hide()
             self._updateTreeview()
@@ -474,8 +462,7 @@ class EffectProperties(Gtk.Expander, Loggable):
 
     def _updateTreeview(self):
         self.storemodel.clear()
-        clip = self.clips[0]
-        for effect in clip.get_top_effects():
+        for effect in self.clip.get_top_effects():
             if effect.props.bin_description in HIDDEN_EFFECTS:
                 continue
             effect_info = self.app.effects.getInfo(effect.props.bin_description)
@@ -541,7 +528,6 @@ class TransformationProperties(Gtk.Expander, Loggable):
         self.source = None
         self._selected_clip = None
         self.spin_buttons = {}
-        self.default_values = {}
         self.set_label(_("Transformation"))
 
         self.builder = Gtk.Builder()
@@ -577,7 +563,7 @@ class TransformationProperties(Gtk.Expander, Loggable):
 
     def _defaultValuesCb(self, unused_widget):
         for name, spinbtn in list(self.spin_buttons.items()):
-            spinbtn.set_value(self.default_values[name])
+            spinbtn.set_value(self.source.ui.default_position[name])
 
     def __sourcePropertyChangedCb(self, unused_source, unused_element, param):
         try:
@@ -594,12 +580,6 @@ class TransformationProperties(Gtk.Expander, Loggable):
         for name, spinbtn in list(self.spin_buttons.items()):
             res, value = self.source.get_child_property(name)
             assert res
-            if name == "width":
-                self.default_values[name] = self._project.videowidth
-            elif name == "height":
-                self.default_values[name] = self._project.videoheight
-            else:
-                self.default_values[name] = 0
             spinbtn.set_value(value)
 
     def __setupSpinButton(self, widget_name, property_name):
@@ -618,9 +598,10 @@ class TransformationProperties(Gtk.Expander, Loggable):
         res, cvalue = self.source.get_child_property(prop)
         assert res
         if value != cvalue:
-            with self.app.action_log.started("Transformation property change"):
+            with self.app.action_log.started("Transformation property change",
+                                             finalizing_action=CommitTimelineFinalizingAction(self._project.pipeline),
+                                             toplevel=True):
                 self.source.set_child_property(prop, value)
-            self._project.pipeline.commit_timeline()
             self.app.gui.viewer.overlay_stack.update(self.source)
 
     def __setSource(self, source):
