@@ -36,12 +36,12 @@ def in_devel():
 LIBDIR = '/usr/lib'
 PKGDATADIR = '/usr/share/pitivi'
 GITVERSION = ''
-VERSION = '0.98'
+VERSION = '0.99'
 APPNAME = 'Pitivi'
 APPURL = 'http://www.pitivi.org/'
 RELEASES_URL = 'http://pitivi.org/releases.txt'
 APPMANUALURL_ONLINE = 'http://www.pitivi.org/manual'
-BUILDDIR = '/build/pitivi/src/pitivi-0.98/mesonbuild'
+BUILDDIR = '/build/pitivi/src/build'
 
 
 if in_devel():
@@ -71,11 +71,6 @@ def get_ui_dir():
     return os.path.join(get_data_dir(), 'ui')
 
 
-def get_renderpresets_dir():
-    """Returns our directory with Render Presets files."""
-    return os.path.join(get_data_dir(), 'renderpresets')
-
-
 def get_audiopresets_dir():
     """Returns our directory with Audio Presets files."""
     return os.path.join(get_data_dir(), 'audiopresets')
