@@ -80,7 +80,6 @@ class ViewerContainer(Gtk.Box, Loggable):
         self.set_border_width(SPACING)
         self.app = app
         self.settings = app.settings
-        self.system = app.system
 
         Loggable.__init__(self)
         self.log("New ViewerContainer")
@@ -111,6 +110,7 @@ class ViewerContainer(Gtk.Box, Loggable):
             pipeline (Pipeline): The Pipeline to switch to.
             position (Optional[int]): The position to seek to initially.
         """
+        self.debug("Setting pipeline: %r", pipeline)
         self._disconnectFromPipeline()
 
         if self.target:
@@ -118,11 +118,7 @@ class ViewerContainer(Gtk.Box, Loggable):
             if parent:
                 parent.remove(self.target)
 
-        self.debug("New pipeline: %r", pipeline)
         self.pipeline = pipeline
-        if position:
-            self.pipeline.simple_seek(position)
-
         self.pipeline.connect("state-change", self._pipelineStateChangedCb)
         self.pipeline.connect("position", self._positionCb)
         self.pipeline.connect("duration-changed", self._durationChangedCb)
@@ -131,6 +127,8 @@ class ViewerContainer(Gtk.Box, Loggable):
         self.__createNewViewer()
         self._setUiActive()
 
+        if position:
+            self.pipeline.simple_seek(position)
         self.pipeline.pause()
 
     def __createNewViewer(self):
@@ -141,8 +139,6 @@ class ViewerContainer(Gtk.Box, Loggable):
 
         if self.docked:
             self.pack_start(self.target, True, True, 0)
-            screen = Gdk.Screen.get_default()
-            height = screen.get_height()
             # Force the AspectFrame to be tall (and wide) enough to look good.
             # TODO: review this code to create a smarter algorithm.
             if not self._compactMode:
@@ -160,11 +156,11 @@ class ViewerContainer(Gtk.Box, Loggable):
         self.target.show_all()
 
     def _disconnectFromPipeline(self):
-        self.debug("Previous pipeline: %r", self.pipeline)
         if self.pipeline is None:
             # silently return, there's nothing to disconnect from
             return
 
+        self.debug("Disconnecting from: %r", self.pipeline)
         self.pipeline.disconnect_by_func(self._pipelineStateChangedCb)
         self.pipeline.disconnect_by_func(self._positionCb)
         self.pipeline.disconnect_by_func(self._durationChangedCb)
@@ -256,6 +252,7 @@ class ViewerContainer(Gtk.Box, Loggable):
         self.timecode_entry.set_tooltip_text(
             _('Enter a timecode or frame number\nand press "Enter" to go to that position'))
         self.timecode_entry.connectActivateEvent(self._entryActivateCb)
+        self.timecode_entry.connect("key_press_event", self._entry_key_press_event_cb)
         bbox.pack_start(self.timecode_entry, False, 10, 0)
 
         self.undock_button = Gtk.ToolButton()
@@ -296,6 +293,11 @@ class ViewerContainer(Gtk.Box, Loggable):
         self.app.project_manager.current_project.pipeline.simple_seek(nanoseconds)
         self.app.gui.timeline_ui.timeline.scrollToPlayhead(align=Gtk.Align.CENTER, when_not_in_view=True)
 
+    def _entry_key_press_event_cb(self, widget, event):
+        """Handles the key press events in the timecode_entry widget."""
+        if event.keyval == Gdk.KEY_Escape:
+            self.app.gui.focusTimeline()
+
     # Active Timeline calllbacks
     def _durationChangedCb(self, unused_pipeline, duration):
         if duration == 0:
@@ -375,6 +377,7 @@ class ViewerContainer(Gtk.Box, Loggable):
         self.docked = True
         self.settings.viewerDocked = True
 
+        position = None
         if self.pipeline:
             position = self.pipeline.getPosition()
             self.pipeline.setState(Gst.State.NULL)
@@ -388,7 +391,7 @@ class ViewerContainer(Gtk.Box, Loggable):
         self.show()
 
         self.external_window.hide()
-        if position:
+        if self.pipeline:
             self.pipeline.pause()
             self.pipeline.simple_seek(position)
 
@@ -441,33 +444,35 @@ class ViewerContainer(Gtk.Box, Loggable):
     def clipTrimPreviewFinished(self):
         """Switches back to the project pipeline following a clip trimming."""
         if self.pipeline is not self.app.project_manager.current_project.pipeline:
+            self.debug("Going back to the project's pipeline")
             self.pipeline.setState(Gst.State.NULL)
             # Using pipeline.getPosition() here does not work because for some
             # reason it's a bit off, that's why we need self._oldTimelinePos.
             self.setPipeline(
                 self.app.project_manager.current_project.pipeline, self._oldTimelinePos)
             self._oldTimelinePos = None
-            self.debug("Back to the project's pipeline")
 
     def _pipelineStateChangedCb(self, unused_pipeline, state, old_state):
         """Updates the widgets when the playback starts or stops."""
-        if int(state) == int(Gst.State.PLAYING):
+        if state == Gst.State.PLAYING:
             st = Gst.Structure.new_empty("play")
             self.app.write_action(st)
             self.playpause_button.setPause()
-            self.system.inhibitScreensaver(self.INHIBIT_REASON)
-        elif int(state) == int(Gst.State.PAUSED):
-            if old_state != int(Gst.State.PAUSED):
-                st = Gst.Structure.new_empty("pause")
-                if old_state == int(Gst.State.PLAYING):
-                    st.set_value("playback_time", float(self.pipeline.getPosition()) /
-                                 Gst.SECOND)
-                self.app.write_action(st)
-
-            self.playpause_button.setPlay()
-            self.system.uninhibitScreensaver(self.INHIBIT_REASON)
+            self.app.simple_inhibit(ViewerContainer.INHIBIT_REASON,
+                                    Gtk.ApplicationInhibitFlags.IDLE)
+            self.overlay_stack.hide_overlays()
         else:
-            self.system.uninhibitScreensaver(self.INHIBIT_REASON)
+            if state == Gst.State.PAUSED:
+                if old_state != Gst.State.PAUSED:
+                    st = Gst.Structure.new_empty("pause")
+                    if old_state == Gst.State.PLAYING:
+                        st.set_value("playback_time",
+                                     self.pipeline.getPosition() / Gst.SECOND)
+                    self.app.write_action(st)
+
+                self.playpause_button.setPlay()
+            self.overlay_stack.show_overlays()
+            self.app.simple_uninhibit(ViewerContainer.INHIBIT_REASON)
 
 
 class ViewerWidget(Gtk.AspectFrame, Loggable):
@@ -477,14 +482,14 @@ class ViewerWidget(Gtk.AspectFrame, Loggable):
         sink_widget (Gtk.Widget): The widget doing the real work.
     """
 
-    def __init__(self, sink_widget):
+    def __init__(self, widget):
         # Prevent black frames and flickering while resizing or changing focus:
         # The aspect ratio gets overridden by setDisplayAspectRatio.
         Gtk.AspectFrame.__init__(self, xalign=0.5, yalign=0.5,
                                  ratio=4.0 / 3.0, obey_child=False)
         Loggable.__init__(self)
 
-        self.add(sink_widget)
+        self.add(widget)
 
         # We keep the ViewerWidget hidden initially, or the desktop wallpaper
         # would show through the non-double-buffered widget!
